@@ -36,6 +36,7 @@
          notify/1,
          notify/2,
          notify/3,
+         tagged_notify/4,
          notify_existing_metric/3,
          get_handlers/0,
          get_handlers_info/0,
@@ -43,7 +44,8 @@
          get_values/1,
          get_history_values/2,
          get_group_values/1,
-         get_group_values/2
+         get_group_values/2,
+         get_tags/1
         ]).
 
 -record(metric, {
@@ -111,12 +113,16 @@ notify(Name, Event) ->
 
 %% notify/3, makes sure metric exist, if not creates metric
 notify(Name, Event, Type) ->
-    case handler_exists(Name) of
-        true ->
-            notify(Name, Event, Type, true);
-        false ->
-            notify(Name, Event, Type, false)
-    end.
+    notify(Name, Event, Type, handler_exists(Name)).
+
+tagged_notify(Name, Event, Type, Tags) ->
+    R = notify(Name, Event, Type),
+    case get_tags(Name) of
+        {error, _, _} -> skip;
+        CurrentTags ->
+            [add_tag(Name, T) || T <- Tags, not sets:is_element(T, CurrentTags)]
+    end,
+    R.
 
 %% assumes metric already exists, bypasses above checks
 notify_existing_metric(Name, Event, Type) ->
@@ -131,8 +137,8 @@ get_handlers_info() ->
 get_info(Name) ->
     case handler_exists(Name) of
         true ->
-            [{_, #metric{type = Type}}] = ets:lookup(?FOLSOM_TABLE, Name),
-            {Name, [{type, Type}]};
+            [{_, #metric{type = Type, tags = Tags}}] = ets:lookup(?FOLSOM_TABLE, Name),
+            {Name, [{type, Type}, {tags, Tags}]};
         false ->
             {error, Name, nonexistent_metric}
     end.
@@ -175,6 +181,15 @@ get_group_values(Tag, Type) ->
     Metrics = ets:match(?FOLSOM_TABLE, {'$1', {metric, '$2', Type, '_'}}),
     [{Name, get_values(Name)} || [Name, Tags] <- Metrics, sets:is_element(Tag, Tags)].
 
+get_tags(Name) ->
+    case handler_exists(Name) of
+        true ->
+            {_, Info} = get_info(Name),
+            proplists:get_value(tags, Info);
+        false ->
+            {error, Name, nonexistent_metric}
+    end.
+
 %%%===================================================================
 %%% Internal functions
 %%%===================================================================
@@ -280,7 +295,7 @@ delete_metric(Name, duration) ->
     true = ets:delete(?FOLSOM_TABLE, Name),
     ok;
 delete_metric(Name, counter) ->
-    true = ets:delete(?COUNTER_TABLE, Name),
+    ok = folsom_metrics_counter:delete(Name),
     true = ets:delete(?FOLSOM_TABLE, Name),
     ok;
 delete_metric(Name, gauge) ->
@@ -300,9 +315,9 @@ delete_metric(Name, meter_reader) ->
 delete_metric(Name, spiral) ->
     #spiral{tid=Tid, server=Pid} = folsom_metrics_spiral:get_value(Name),
     folsom_sample_slide_server:stop(Pid),
-    true = ets:delete(Tid),
     ets:delete(?SPIRAL_TABLE, Name),
     ets:delete(?FOLSOM_TABLE, Name),
+    ets:delete(Tid),
     ok.
 
 delete_histogram(Name, #histogram{type = uniform, sample = #uniform{reservoir = Reservoir}}) ->
@@ -315,6 +330,11 @@ delete_histogram(Name, #histogram{type = none, sample = #none{reservoir = Reserv
     true = ets:delete(?FOLSOM_TABLE, Name),
     true = ets:delete(Reservoir),
     ok;
+delete_histogram(Name, #histogram{type = slide_sorted, sample = #slide_sorted{reservoir = Reservoir}}) ->
+    true = ets:delete(?HISTOGRAM_TABLE, Name),
+    true = ets:delete(?FOLSOM_TABLE, Name),
+    true = ets:delete(Reservoir),
+    ok;
 delete_histogram(Name, #histogram{type = exdec}) ->
     true = ets:delete(?HISTOGRAM_TABLE, Name),
     true = ets:delete(?FOLSOM_TABLE, Name),
